# -*- coding: utf-8 -*-


#http://localization-guide.readthedocs.org/en/latest/l10n/pluralforms.html
__plural_forms = {}

#Acholi; nplurals=2; plural = (n > 1);
__plural_forms['ach'] = lambda n:(n > 1)

#Afrikaans; nplurals=2; plural = (n != 1);
__plural_forms['af'] = lambda n:(n != 1)

#Akan; nplurals=2; plural = (n > 1);
__plural_forms['ak'] = lambda n:(n > 1)

#Amharic; nplurals=2; plural = (n > 1);
__plural_forms['am'] = lambda n:(n > 1)

#Aragonese; nplurals=2; plural = (n != 1);
__plural_forms['an'] = lambda n:(n != 1)

#Angika; nplurals=2; plural = (n != 1);
__plural_forms['anp'] = lambda n:(n != 1)

#Arabic; nplurals=6; plural = (n == 0 ? 0 : n == 1 ? 1 : n == 2 ? 2 : n % 100 >= 3 && n % 100 <= 10 ? 3 : n % 100 >= 11 ? 4 : 5);
__plural_forms['ar'] = lambda n:(0 if n == 0 else 1 if n == 1 else 2 if n == 2 else 3 if n % 100 >= 3 and n % 100 <= 10 else 4 if n % 100 >= 11 else 5)

#Mapudungun; nplurals=2; plural = (n > 1);
__plural_forms['arn'] = lambda n:(n > 1)

#Assamese; nplurals=2; plural = (n != 1);
__plural_forms['as'] = lambda n:(n != 1)

#Asturian; nplurals=2; plural = (n != 1);
__plural_forms['ast'] = lambda n:(n != 1)

#Aymará; nplurals=1; plural = 0;
__plural_forms['ay'] = lambda n:0

#Azerbaijani; nplurals=2; plural = (n != 1);
__plural_forms['az'] = lambda n:(n != 1)

#Belarusian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['be'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Bulgarian; nplurals=2; plural = (n != 1);
__plural_forms['bg'] = lambda n:(n != 1)

#Bengali; nplurals=2; plural = (n != 1);
__plural_forms['bn'] = lambda n:(n != 1)

#Tibetan; nplurals=1; plural = 0;
__plural_forms['bo'] = lambda n:0

#Breton; nplurals=2; plural = (n > 1);
__plural_forms['br'] = lambda n:(n > 1)

#Bodo; nplurals=2; plural = (n != 1);
__plural_forms['brx'] = lambda n:(n != 1)

#Bosnian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['bs'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Catalan; nplurals=2; plural = (n != 1);
__plural_forms['ca'] = lambda n:(n != 1)

#Chiga; nplurals=1; plural = 0;
__plural_forms['cgg'] = lambda n:0

#Czech; nplurals=3; plural = (n == 1) ? 0 : (n >= 2 && n <= 4) ? 1 : 2;
__plural_forms['cs'] = lambda n:(0 if n == 1 else 1 if (n >= 2 and n <= 4) else 2)

#Kashubian; nplurals=3; plural = (n == 1) ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2;
__plural_forms['csb'] = lambda n:(0 if n == 1 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Welsh; nplurals=4; plural = (n == 1) ? 0 : (n == 2) ? 1 : (n != 8 && n != 11) ? 2 : 3;
__plural_forms['cy'] = lambda n:(0 if n == 1 else 1 if n == 2 else 2 if (n != 8 and n != 11) else 3)

#Danish; nplurals=2; plural = (n != 1);
__plural_forms['da'] = lambda n:(n != 1)

#German; nplurals=2; plural = (n != 1);
__plural_forms['de'] = lambda n:(n != 1)

#Dogri; nplurals=2; plural = (n != 1);
__plural_forms['doi'] = lambda n:(n != 1)

#Dzongkha; nplurals=1; plural = 0;
__plural_forms['dz'] = lambda n:0

#Greek; nplurals=2; plural = (n != 1);
__plural_forms['el'] = lambda n:(n != 1)

#English; nplurals=2; plural = (n != 1);
__plural_forms['en'] = lambda n:(n != 1)

#Esperanto; nplurals=2; plural = (n != 1);
__plural_forms['eo'] = lambda n:(n != 1)

#Spanish; nplurals=2; plural = (n != 1);
__plural_forms['es'] = lambda n:(n != 1)

#Argentinean; Spanish nplurals=2; plural = (n != 1);
__plural_forms['es_AR'] = lambda n:(n != 1)

#Estonian; nplurals=2; plural = (n != 1);
__plural_forms['et'] = lambda n:(n != 1)

#Basque; nplurals=2; plural = (n != 1);
__plural_forms['eu'] = lambda n:(n != 1)

#Persian; nplurals=1; plural = 0;
__plural_forms['fa'] = lambda n:0

#Fulah; nplurals=2; plural = (n != 1);
__plural_forms['ff'] = lambda n:(n != 1)

#Finnish; nplurals=2; plural = (n != 1);
__plural_forms['fi'] = lambda n:(n != 1)

#Filipino; nplurals=2; plural = (n > 1);
__plural_forms['fil'] = lambda n:(n > 1)

#Faroese; nplurals=2; plural = (n != 1);
__plural_forms['fo'] = lambda n:(n != 1)

#French; nplurals=2; plural = (n > 1);
__plural_forms['fr'] = lambda n:(n > 1)

#Friulian; nplurals=2; plural = (n != 1);
__plural_forms['fur'] = lambda n:(n != 1)

#Frisian; nplurals=2; plural = (n != 1);
__plural_forms['fy'] = lambda n:(n != 1)

#Irish; nplurals=5; plural = n == 1 ? 0 : n == 2 ? 1 : (n > 2 && n < 7) ? 2 :(n > 6 && n < 11) ? 3 : 4;
__plural_forms['ga'] = lambda n:(0 if n == 1 else 1 if n == 2 else 2 if (n > 2 and n < 7) else 3 if (n > 6 and n < 11) else 4)

#Scottish; Gaelic nplurals=4; plural = (n == 1 || n == 11) ? 0 : (n == 2 || n == 12) ? 1 : (n > 2 && n < 20) ? 2 : 3;
__plural_forms['gd'] = lambda n:(0 if (n == 1 or n == 11) else 1 if (n == 2 or n == 12) else 2 if (n > 2 and n < 20) else 3)

#Galician; nplurals=2; plural = (n != 1);
__plural_forms['gl'] = lambda n:(n != 1)

#Gujarati; nplurals=2; plural = (n != 1);
__plural_forms['gu'] = lambda n:(n != 1)

#Gun; nplurals=2; plural = (n > 1);
__plural_forms['gun'] = lambda n:(n > 1)

#Hausa; nplurals=2; plural = (n != 1);
__plural_forms['ha'] = lambda n:(n != 1)

#Hebrew; nplurals=2; plural = (n != 1);
__plural_forms['he'] = lambda n:(n != 1)

#Hindi; nplurals=2; plural = (n != 1);
__plural_forms['hi'] = lambda n:(n != 1)

#Chhattisgarhi; nplurals=2; plural = (n != 1);
__plural_forms['hne'] = lambda n:(n != 1)

#Croatian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
#__plural_forms['hr'] = lambda n:(n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2)
__plural_forms['hr'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Hungarian; nplurals=2; plural = (n != 1);
__plural_forms['hu'] = lambda n:(n != 1)

#Armenian; nplurals=2; plural = (n != 1);
__plural_forms['hy'] = lambda n:(n != 1)

#erlingua; nplurals=2; plural = (n != 1);
__plural_forms['ia'] = lambda n:(n != 1)

#Indonesian; nplurals=1; plural = 0;
__plural_forms['id'] = lambda n:0

#Icelandic; nplurals=2; plural = (n % 10 != 1 || n % 100 == 11);
__plural_forms['is'] = lambda n:(n % 10 != 1 or n % 100 == 11)

#Italian; nplurals=2; plural = (n != 1);
__plural_forms['it'] = lambda n:(n != 1)

#Japanese; nplurals=1; plural = 0;
__plural_forms['ja'] = lambda n:0

#Lojban; nplurals=1; plural = 0;
__plural_forms['jbo'] = lambda n:0

#Javanese; nplurals=2; plural = (n != 0);
__plural_forms['jv'] = lambda n:(n != 0)

#Georgian; nplurals=1; plural = 0;
__plural_forms['ka'] = lambda n:0

#Kazakh; nplurals=1; plural = 0;
__plural_forms['kk'] = lambda n:0

#Greenlandic; nplurals=2; plural = (n != 1);
__plural_forms['kl'] = lambda n:(n != 1)

#Khmer; nplurals=1; plural = 0;
__plural_forms['km'] = lambda n:0

#Kannada; nplurals=2; plural = (n != 1);
__plural_forms['kn'] = lambda n:(n != 1)

#Korean; nplurals=1; plural = 0;
__plural_forms['ko'] = lambda n:0

#Kurdish; nplurals=2; plural = (n != 1);
__plural_forms['ku'] = lambda n:(n != 1)

#Cornish; nplurals=4; plural = (n == 1) ? 0 : (n == 2) ? 1 : (n  ==  3) ? 2 : 3;
__plural_forms['kw'] = lambda n:(0 if (n == 1) else 1 if (n == 2) else 2 if (n == 3) else 3)

#Kyrgyz; nplurals=1; plural = 0;
__plural_forms['ky'] = lambda n:0

#Letzeburgesch; nplurals=2; plural = (n != 1);
__plural_forms['lb'] = lambda n:(n != 1)

#Lingala; nplurals=2; plural = (n > 1);
__plural_forms['ln'] = lambda n:(n > 1)

#Lao; nplurals=1; plural = 0;
__plural_forms['lo'] = lambda n:0

#Lithuanian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['lt'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Latvian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n != 0 ? 1 : 2);
__plural_forms['lv'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n != 0 else 2)

#Maithili; nplurals=2; plural = (n != 1);
__plural_forms['mai'] = lambda n:(n != 1)

#Mauritian; Creole nplurals=2; plural = (n > 1);
__plural_forms['mfe'] = lambda n:(n > 1)

#Malagasy; nplurals=2; plural = (n > 1);
__plural_forms['mg'] = lambda n:(n > 1)

#Maori; nplurals=2; plural = (n > 1);
__plural_forms['mi'] = lambda n:(n > 1)

#Macedonian; nplurals=2; plural =  n == 1 || n % 10 == 1 ? 0 : 1;
#__plural_forms['mk'] = lambda n: n == 1 or n % 10 == 1 ? 0 : 1 #Can’t be correct needs a 2 somewhere
__plural_forms['mk'] = lambda n:(0 if n == 1 or n % 10 == 1 else 1)

#Malayalam; nplurals=2; plural = (n != 1);
__plural_forms['ml'] = lambda n:(n != 1)

#Mongolian; nplurals=2; plural = (n != 1);
__plural_forms['mn'] = lambda n:(n != 1)

#Manipuri; nplurals=2; plural = (n != 1);
__plural_forms['mni'] = lambda n:(n != 1)

#Mandinka; nplurals=3; plural = (n == 0 ? 0 : n == 1 ? 1 : 2);
__plural_forms['mnk'] = lambda n:(0 if n == 0 else 1 if n == 1 else 2)

#Marathi; nplurals=2; plural = (n != 1);
__plural_forms['mr'] = lambda n:(n != 1)

#Malay; nplurals=1; plural = 0;
__plural_forms['ms'] = lambda n:0

#Maltese; nplurals=4; plural = (n == 1 ? 0 : n == 0 || ( n % 100>1 && n % 100 < 11) ? 1 : (n % 100>10 && n % 100 < 20 ) ? 2 : 3);
__plural_forms['mt'] = lambda n:(0 if n == 1 else 1 if n == 0 or (n % 100 > 1 and n % 100 < 11) else 2 if (n % 100 > 10 and n % 100 < 20) else 3)

#Burmese; nplurals=1; plural = 0;
__plural_forms['my'] = lambda n:0

#Nahuatl; nplurals=2; plural = (n != 1);
__plural_forms['nah'] = lambda n:(n != 1)

#Neapolitan; nplurals=2; plural = (n != 1);
__plural_forms['nap'] = lambda n:(n != 1)

#Norwegian; Bokmal nplurals=2; plural = (n != 1);
__plural_forms['nb'] = lambda n:(n != 1)

#Nepali; nplurals=2; plural = (n != 1);
__plural_forms['ne'] = lambda n:(n != 1)

#Dutch; nplurals=2; plural = (n != 1);
__plural_forms['nl'] = lambda n:(n != 1)

#Norwegian; Nynorsk nplurals=2; plural = (n != 1);
__plural_forms['nn'] = lambda n:(n != 1)

#Norwegian; (old code) nplurals=2; plural = (n != 1);
__plural_forms['no'] = lambda n:(n != 1)

#Northern; Sotho nplurals=2; plural = (n != 1);
__plural_forms['nso'] = lambda n:(n != 1)

#Occitan; nplurals=2; plural = (n > 1);
__plural_forms['oc'] = lambda n:(n > 1)

#Oriya; nplurals=2; plural = (n != 1);
__plural_forms['or'] = lambda n:(n != 1)

#Punjabi; nplurals=2; plural = (n != 1);
__plural_forms['pa'] = lambda n:(n != 1)

#Papiamento; nplurals=2; plural = (n != 1);
__plural_forms['pap'] = lambda n:(n != 1)

#Polish; nplurals=3; plural = (n == 1 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['pl'] = lambda n:(0 if n == 1 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Piemontese; nplurals=2; plural = (n != 1);
__plural_forms['pms'] = lambda n:(n != 1)

#Pashto; nplurals=2; plural = (n != 1);
__plural_forms['ps'] = lambda n:(n != 1)

#Portuguese; nplurals=2; plural = (n != 1);
__plural_forms['pt'] = lambda n:(n != 1)

#Brazilian; Portuguese nplurals=2; plural = (n > 1);
__plural_forms['pt_BR'] = lambda n:(n > 1)

#Romansh; nplurals=2; plural = (n != 1);
__plural_forms['rm'] = lambda n:(n != 1)

#Romanian; nplurals=3; plural = (n == 1 ? 0 : (n == 0 || (n % 100 > 0 && n % 100 < 20)) ? 1 : 2);
__plural_forms['ro'] = lambda n:(0 if n == 1 else 1 if (n == 0 or (n % 100 > 0 and n % 100 < 20)) else 2)

#Russian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['ru'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Kinyarwanda; nplurals=2; plural = (n != 1);
__plural_forms['rw'] = lambda n:(n != 1)

#Yakut; nplurals=1; plural = 0;
__plural_forms['sah'] = lambda n:0

#Santali; nplurals=2; plural = (n != 1);
__plural_forms['sat'] = lambda n:(n != 1)

#Scots; nplurals=2; plural = (n != 1);
__plural_forms['sco'] = lambda n:(n != 1)

#Sindhi; nplurals=2; plural = (n != 1);
__plural_forms['sd'] = lambda n:(n != 1)

#Northern; Sami nplurals=2; plural = (n != 1);
__plural_forms['se'] = lambda n:(n != 1)

#Sinhala; nplurals=2; plural = (n != 1);
__plural_forms['si'] = lambda n:(n != 1)

#Slovak; nplurals=3; plural = (n == 1) ? 0 : (n >= 2 && n <= 4) ? 1 : 2;
__plural_forms['sk'] = lambda n:(0 if n == 1 else 1 if (n >= 2 and n <= 4) else 2)

#Slovenian; nplurals=4; plural = (n % 100 == 1 ? 1 : n % 100 == 2 ? 2 : n % 100 == 3 || n % 100 == 4 ? 3 : 0);
__plural_forms['sl'] = lambda n:(1 if n % 100 == 1 else 2 if n % 100 == 2 else 3 if n % 100 == 3 or n % 100 == 4 else 0)

#Somali; nplurals=2; plural = (n != 1);
__plural_forms['so'] = lambda n:(n != 1)

#Songhay; nplurals=2; plural = (n != 1);
__plural_forms['son'] = lambda n:(n != 1)

#Albanian; nplurals=2; plural = (n != 1);
__plural_forms['sq'] = lambda n:(n != 1)

#Serbian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['sr'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Sundanese; nplurals=1; plural = 0;
__plural_forms['su'] = lambda n:0

#Swedish; nplurals=2; plural = (n != 1);
__plural_forms['sv'] = lambda n:(n != 1)

#Swahili; nplurals=2; plural = (n != 1);
__plural_forms['sw'] = lambda n:(n != 1)

#Tamil; nplurals=2; plural = (n != 1);
__plural_forms['ta'] = lambda n:(n != 1)

#Telugu; nplurals=2; plural = (n != 1);
__plural_forms['te'] = lambda n:(n != 1)

#Tajik; nplurals=2; plural = (n > 1);
__plural_forms['tg'] = lambda n:(n > 1)

#Thai; nplurals=1; plural = 0;
__plural_forms['th'] = lambda n:0

#Tigrinya; nplurals=2; plural = (n > 1);
__plural_forms['ti'] = lambda n:(n > 1)

#Turkmen; nplurals=2; plural = (n != 1);
__plural_forms['tk'] = lambda n:(n != 1)

#Turkish; nplurals=2; plural = (n > 1);
__plural_forms['tr'] = lambda n:(n > 1)

#Tatar; nplurals=1; plural = 0;
__plural_forms['tt'] = lambda n:0

#Uyghur; nplurals=1; plural = 0;
__plural_forms['ug'] = lambda n:0

#Ukrainian; nplurals=3; plural = (n % 10 == 1 && n % 100 != 11 ? 0 : n % 10 >= 2 && n % 10 <= 4 && (n % 100 < 10 || n % 100 >= 20) ? 1 : 2);
__plural_forms['uk'] = lambda n:(0 if n % 10 == 1 and n % 100 != 11 else 1 if n % 10 >= 2 and n % 10 <= 4 and (n % 100 < 10 or n % 100 >= 20) else 2)

#Urdu; nplurals=2; plural = (n != 1);
__plural_forms['ur'] = lambda n:(n != 1)

#Uzbek; nplurals=2; plural = (n > 1);
__plural_forms['uz'] = lambda n:(n > 1)

#Vietnamese; nplurals=1; plural = 0;
__plural_forms['vi'] = lambda n:0

#Walloon; nplurals=2; plural = (n > 1);
__plural_forms['wa'] = lambda n:(n > 1)

#Wolof; nplurals=1; plural = 0;
__plural_forms['wo'] = lambda n:0

#Yoruba; nplurals=2; plural = (n != 1);
__plural_forms['yo'] = lambda n:(n != 1)

#Chinese; [2] nplurals=1; plural = 0;
__plural_forms['zh'] = lambda n:0

#Chinese; [3] nplurals=2; plural = (n > 1);
__plural_forms['zh'] = lambda n:(n > 1)


def get_plural_forms():
    
    return __plural_forms.copy()
    

def get_plural_form(language_code, n):
    
    plural_key = language_code.lower()
    plural_form = __plural_forms.get(plural_key, None)
    
    if not plural_form:
        
        plural_key = plural_key.split('-')[0]
        plural_form = __plural_forms.get(plural_key, None)
        
        if not plural_form:
            
            plural_key = plural_key.split('_')[0]
            plural_form = __plural_forms.get(plural_key, None)
            
            if not plural_form:
                raise ValueError('Invalid/Unsupported language code: %s' % (language_code, ))
                
    return int(plural_form( n ))
    
    


# -*- coding: utf-8 -*-

from django.core.cache import get_cache

from kway import settings, utils
    

def get_value_for_key(key, default_value = None):
    
    cache = get_cache(settings.KWAY_CACHE_NAME)
    localized_key = utils.get_localized_key(key)
    
    value = None
    
    if cache:
        
        value = cache.get(localized_key, None)
        
        if value:
            
            cache.set(localized_key, value)
        
        cache.close()
            
    return value or default_value
    
    
def set_value_for_key(key, value):
    
    cache = get_cache(settings.KWAY_CACHE_NAME)
    localized_key = utils.get_localized_key(key)
    
    if cache:
        
        if value:
            cache.set(localized_key, value)
        else:
            cache.delete(localized_key)
        
        cache.close()
        
    return value
    
    
def update_values_post_save(sender, instance, **kwargs):
    
    if kwargs['created']:
        return
        
    cache = get_cache(settings.KWAY_CACHE_NAME)
    
    if cache:
        
        for language in settings.KWAY_LANGUAGES:
            
            language_code = language[0]
            localized_key = utils.get_localized_key(instance.key, language_code)
            localized_value_field_name = utils.get_localized_value_field_name(language_code)
            localized_value = getattr(instance, localized_value_field_name)
            
            cache.set(localized_key, localized_value)
            
        cache.close()
        
        


# -*- coding: utf-8 -*-

from django.contrib import admin
from django.db import models
from django.db.models import Q
from django.forms import Textarea

from kway.models import KImage, KText
from kway import settings, utils


__KTextAdminBaseClass = admin.ModelAdmin

if settings.KWAY_USE_MODELTRANSLATION:

    try:
        from modeltranslation.admin import TabbedTranslationAdmin as __KTextAdminBaseClass

    except ImportError:
        pass
    

__KImageAdminImageMixin = object
get_thumbnail = None

if settings.KWAY_USE_SORL_THUMBNAIL:

    try:
        from sorl.thumbnail.admin import AdminImageMixin as __KImageAdminImageMixin
        from sorl.thumbnail import get_thumbnail as get_thumbnail
        
    except ImportError:
        pass
    
    
class BlankListFilter(admin.SimpleListFilter):
    
    title = 'blank'
    parameter_name = 'blank'

    def lookups(self, request, model_admin):
        
        lookups_values = ()
        
        for language in settings.KWAY_LANGUAGES:
            
            lookups_values += ((language[0], language[1], ), )
            
        return lookups_values
        
    def queryset(self, request, queryset):
        
        value = self.value()
        
        if value:
            
            value_key = utils.get_localized_value_field_name(self.value())
            
            value_isnull = {}
            value_isnull[ value_key + '__isnull' ] = True
            
            value_isempty = {}
            value_isempty[ value_key + '__exact' ] = ''
            
            return queryset.filter(Q( **value_isnull ) | Q( **value_isempty ))
        
        else:
            return queryset
            

class KImageAdmin(__KImageAdminImageMixin, admin.ModelAdmin):
    
    actions = None
    
    list_display = ('key', 'value', )
    
    if get_thumbnail:
        
        def value_thumbnail(self, obj):
            
            thumbnail_width = 120
            thumbnail_height = 80
            thumbnail_html = ''
            
            if obj.value:
            
                try:
                    thumbnail = get_thumbnail(obj.value, '%sx%s' % (thumbnail_width, thumbnail_height, ))
                    thumbnail_html = '<img src="%s" width="%s" height="%s" />' % (thumbnail.url, thumbnail_width, thumbnail_height, )
                
                except IOError:
                    pass
            
            return thumbnail_html
        
        value_thumbnail.allow_tags = True
        value_thumbnail.short_description = 'Value Preview'
        
        list_display += ('value_thumbnail', )
        
    list_per_page = settings.KWAY_ADMIN_LIST_PER_PAGE
    
    readonly_fields = ('key', )
    
    def get_readonly_fields(self, request, obj = None):
        return () if request.user.is_superuser else self.readonly_fields
    
    search_fields = ('key', )
    
    fieldsets = (
        (None, {
            'classes': ('wide', ),
            'fields': ('key', 'value', )
        }),
    )
    
    save_on_top = True
    
    def has_add_permission(self, request, obj = None):
        return request.user.is_superuser
        
    def has_delete_permission(self, request, obj = None):
        return request.user.is_superuser
    
admin.site.register(KImage, KImageAdmin)


class KTextAdmin(__KTextAdminBaseClass):
    
    actions = None
    
    value_fields = ()
    
    for language in settings.KWAY_LANGUAGES:
        value_fields += (utils.get_localized_value_field_name(language[0]), )
        
    list_display = ('key', ) + value_fields
    
    if settings.KWAY_ADMIN_LIST_EDITABLE:
        list_editable = value_fields
    
    list_filter = (BlankListFilter, )
    list_per_page = settings.KWAY_ADMIN_LIST_PER_PAGE
    
    readonly_fields = ('key', )
    
    def get_readonly_fields(self, request, obj = None):
        return () if request.user.is_superuser else self.readonly_fields
    
    search_fields = ('key', ) + value_fields
    
    fieldsets = (
        (None, {
            'classes': ('wide', ),
            'fields': ('key', ) + value_fields
        }),
    )
    
    save_on_top = True
    
    def has_add_permission(self, request, obj = None):
        return request.user.is_superuser
        
    def has_delete_permission(self, request, obj = None):
        return request.user.is_superuser
        
    formfield_overrides = {
        models.TextField:{ 'widget':Textarea( attrs = { 'rows':3, 'cols':25 } ) },
    }
    
admin.site.register(KText, KTextAdmin)




# -*- coding: utf-8 -*-

from django.db import models
from django.db.models import ImageField
from django.db.models.signals import post_save

from kway import cache, settings, utils


__add_localized_value_fields_manually = True

if settings.KWAY_USE_MODELTRANSLATION:
    
    try:
        from modeltranslation.translator import translator
        
        __add_localized_value_fields_manually = False
        
    except ImportError:
        pass


if settings.KWAY_USE_SORL_THUMBNAIL:
    
    try:
        from sorl.thumbnail import ImageField
        
    except ImportError:    
        pass
    

class KImage(models.Model):
    
    key = models.CharField(max_length = 100, unique = True, verbose_name = 'Key')
    value = ImageField(blank = True, default = '', upload_to = 'uploads/kway/images/', verbose_name = 'Value')
    
    class Meta:
        app_label = 'kway'
        ordering = ['key']
        verbose_name = 'Image'
        verbose_name_plural = 'Images'
        
    def __unicode__(self):
        
        return unicode(u'[%s] - %s' % (self.key, self.value, ))


class KText(models.Model):
    
    key = models.CharField(max_length = 100, unique = True, verbose_name = 'Key')
    value = models.TextField(blank = True, default = '', verbose_name = 'Value')
    
    class Meta:
        app_label = 'kway'
        ordering = ['key']
        verbose_name = 'Text'
        verbose_name_plural = 'Texts'
        
    def __unicode__(self):
        
        return unicode(u'[%s] - %s' % (self.key, self.value, ))

        
if __add_localized_value_fields_manually:
    
    #http://www.mixedcase.nl/articles/2009/11/26/how-dynamically-add-fields-django-model/
    for language in settings.KWAY_LANGUAGES:
        
        value_field_name = utils.get_localized_value_field_name(language[0])
        value_field_obj = models.TextField(blank = True, default = '', verbose_name = 'Value [%s]' % language[0])
        
        KText.add_to_class(value_field_name, value_field_obj)
    
    
post_save.connect(cache.update_values_post_save, sender = KText)




# -*- coding: utf-8 -*-

from django.utils import translation

from kway import cache
from kway import localization
from kway import settings
from kway import utils
from kway.models import KImage, KText
from kway.version import __version__


def kgetimage( key, *args, **kwargs ):

    obj, obj_created = KImage.objects.get_or_create(key = key)

    return obj.value.url if obj.value else ''


def kgettext( key, key_plural = None, count = None, *args, **kwargs ):
    
    value_args = kwargs.copy()
    
    if 'default' in value_args:
        del(value_args['default'])
        
    if count != None:
        
        count = int(count)
        
        if not 'count' in value_args:
            value_args['count'] = count
        
        if not 'n' in value_args:
            value_args['n'] = count
            
        abs_count = (count if count >= 0 else -count)
        
        plural_form = localization.get_plural_form(translation.get_language(), abs_count)
        
        key = key_plural if key_plural and plural_form > 0 else key
        
        try:
            key = key % str(plural_form)
        
        except KeyError:
            pass
            
        except ValueError:
            pass
        
    value = cache.get_value_for_key(key)

    key_value = (key if settings.KWAY_USE_KEY_AS_VALUE else '')
    
    default_value = kwargs.get('default', (key if settings.KWAY_USE_KEY_AS_DEFAULT_VALUE else ''))
    
    debug_value = ('[[ ' + key + ' ]]' if settings.KWAY_USE_KEY_AS_DEBUG_VALUE else '')
    
    if value == None:
        
        default_language_localized_value_field_name = utils.get_localized_value_field_name( settings.KWAY_LANGUAGES[ settings.KWAY_DEFAULT_LANGUAGE ][0] )
        current_language_localized_value_field_name = utils.get_localized_value_field_name()
        
        obj_defaults = { default_language_localized_value_field_name:default_value }
        
        obj, obj_created = KText.objects.get_or_create(key = key, defaults = obj_defaults)
        
        value = getattr(obj, current_language_localized_value_field_name, getattr(obj, default_language_localized_value_field_name, default_value))
        
        cache.set_value_for_key(key, value)
        
    value = (value or key_value or debug_value)
    
    if len(value_args.keys()):
        
        try:
            value = value % value_args
        
        except KeyError:
            pass
            
        except ValueError:
            pass
            
    return value
    
    


# -*- coding: utf-8 -*-

from django.utils import translation

from kway import settings


def get_localized_key(key, language_code = None):
    
    language_code = (language_code or translation.get_language())
    language_key = language_code.replace('-', '_')
    
    localized_key = key + '_' + language_key
    
    return localized_key
    
    
__get_localized_value_field_name_proxy = get_localized_key

if settings.KWAY_USE_MODELTRANSLATION:
    
    try:
        from modeltranslation.utils import build_localized_fieldname as __get_localized_value_field_name_proxy
        
    except ImportError:
        pass
        
        
def get_localized_value_field_name(language_code = None):
    
    language_code = (language_code or translation.get_language())
    
    return __get_localized_value_field_name_proxy('value', language_code)
    
    


# -*- coding: utf-8 -*-

from django.conf import settings


KWAY_ADMIN_LIST_EDITABLE = getattr(settings, 'KWAY_ADMIN_LIST_EDITABLE', True)
KWAY_ADMIN_LIST_PER_PAGE = getattr(settings, 'KWAY_ADMIN_LIST_PER_PAGE', 100)

KWAY_CACHE_NAME = getattr(settings, 'KWAY_CACHE_NAME', 'kway')
KWAY_CACHE_KEY_PREFIX = getattr(settings, 'KWAY_CACHE_KEY_PREFIX', 'kway')
KWAY_CACHE_TIMEOUT =  getattr(settings, 'KWAY_CACHE_TIMEOUT', (60 * 60 * 24)) #1 day

__cache_backend = settings.CACHES.get(KWAY_CACHE_NAME, None)

if not __cache_backend:
    
    __cache_backend = {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache', 
        'KEY_PREFIX': KWAY_CACHE_KEY_PREFIX, 
        'TIMEOUT': KWAY_CACHE_TIMEOUT
    }
    
    settings.CACHES[KWAY_CACHE_NAME] = __cache_backend
    
KWAY_LANGUAGES = getattr(settings, 'KWAY_LANGUAGES', settings.LANGUAGES)

if len(KWAY_LANGUAGES) > 0:
    for language in KWAY_LANGUAGES:
        if not language in settings.LANGUAGES:
            raise ValueError('KWAY_LANGUAGES cannot contain invalid languages: %s' % str(language))
else:
    raise ValueError('KWAY_LANGUAGES must contain at least 1 language')
    
KWAY_DEFAULT_LANGUAGE = getattr(settings, 'KWAY_DEFAULT_LANGUAGE', settings.DEFAULT_LANGUAGE)
KWAY_LANGUAGES[KWAY_DEFAULT_LANGUAGE]

KWAY_USE_KEY_AS_DEBUG_VALUE = getattr(settings, 'KWAY_USE_KEY_AS_DEBUG_VALUE', True) and settings.DEBUG
KWAY_USE_KEY_AS_DEFAULT_VALUE = getattr(settings, 'KWAY_USE_KEY_AS_DEFAULT_VALUE', False)
KWAY_USE_KEY_AS_VALUE = getattr(settings, 'KWAY_USE_KEY_AS_VALUE', False)

KWAY_USE_MODELTRANSLATION = ('modeltranslation' in settings.INSTALLED_APPS)
KWAY_USE_SORL_THUMBNAIL = ('sorl.thumbnail' in settings.INSTALLED_APPS)




# -*- coding: utf-8 -*-

from django import template
from django.template.defaultfilters import stringfilter

from kway import kgetimage, kgettext


register = template.Library()


@register.simple_tag(takes_context = False)
def kimage(key, *args, **kwargs):
    
    value = kgetimage(key, *args, **kwargs)
    
    return value


@register.simple_tag(takes_context = False)
def ktext(key, key_plural = None, count = None, *args, **kwargs):
    
    value = kgettext(key, key_plural, count, *args, **kwargs)
    
    return value
    
    
@register.simple_tag(takes_context = False)
def ktrans(key, key_plural = None, count = None, *args, **kwargs):
    
    value = kgettext(key, key_plural, count, *args, **kwargs)
    
    return value
    
    



#!/usr/bin/env python
from setuptools import setup, find_packages

exec(open('kway/version.py').read())

setup(
    name='django-kway',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    version=__version__,
    description='django-kway is an alternative to django gettext catalog with admin integration.',
    long_description=(
        'django-kway is an alternative to django gettext catalog and offers: '
        'i18n support (with multiple plural forms), django admin integration, '
        'kgettext function, ktrans template tag, caching, lazy modeltranslation integration and various settings.'
    ),
    author='Fabio Caccamo',
    author_email='fabio.caccamo@gmail.com',
    url='https://github.com/fabiocaccamo/django-kway',
    download_url='https://github.com/fabiocaccamo/django-kway/archive/%s.tar.gz' % __version__,
    keywords = ['django', 'admin', 'i18n', 'gettext', 'trans', 'alternative', 'key', 'modeltranslation', 'rosetta'],
    requires=['django(>=1.4)'],
    classifiers=[]
)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import sorl.thumbnail.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='KImage',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(unique=True, max_length=100, verbose_name=b'Key')),
                ('value', sorl.thumbnail.fields.ImageField(default=b'', upload_to=b'uploads/kway/images/', verbose_name=b'Value', blank=True)),
            ],
            options={
                'ordering': ['key'],
                'verbose_name': 'Image',
                'verbose_name_plural': 'Images',
            },
        ),
        migrations.CreateModel(
            name='KText',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('key', models.CharField(unique=True, max_length=100, verbose_name=b'Key')),
                ('value', models.TextField(default=b'', verbose_name=b'Value', blank=True)),
            ],
            options={
                'ordering': ['key'],
                'verbose_name': 'Text',
                'verbose_name_plural': 'Texts',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from kway import settings
from kway.models import KText

if settings.KWAY_USE_MODELTRANSLATION:

    try: 
        
        from modeltranslation.translator import translator, TranslationOptions
        
        
        class KTextTranslationOptions(TranslationOptions):
            
            if settings.KWAY_USE_MODELTRANSLATION:
                fields = ('value', )
            else:
                fields = ()
                
        translator.register(KText, KTextTranslationOptions)
        
        
    except ImportError:
        pass

    